@@ -39,6 +39,10 @@ def apply_plist_template(plist_file, version):
     for line in fileinput.input(plist_file, inplace=1):
         print (line.rstrip().replace('${VERSION}', version))
 
+def create_bundle_symlink(binary_name, symlink_name):
+    os.symlink(os.path.basename(binary_name),
+               os.path.join(target_directory(binary_name), symlink_name))
+
 def bundle_version():
     if os.path.exists('VERSION'):
         x = open('VERSION')
@@ -69,6 +73,8 @@ def main():
     copy_bundle(binary_name)
     print("> copying binary")
     copy_binary(binary_name)
+    print("> create bundle symlink")
+    create_bundle_symlink(binary_name, "mpv-bundle")
     print("> generating Info.plist")
     apply_plist_template(target_plist(binary_name), version)
 
