@@ -64,9 +64,6 @@ bool af_fmt_is_pcm(int format);
 int af_fmt_to_planar(int format);
 int af_fmt_from_planar(int format);
 
-// Amount of bytes that contain audio of the given duration, aligned to frames.
-int af_fmt_seconds_to_bytes(int format, float seconds, int channels, int samplerate);
-
 void af_fill_silence(void *dst, size_t bytes, int format);
 
 void af_get_best_sample_formats(int src_format, int *out_formats);
