@@ -35,12 +35,12 @@ struct priv {
     SLBufferQueueItf buffer_queue;
     SLEngineItf engine;
     SLPlayItf play;
-    char *curbuf, *buf1, *buf2;
+    char *buf;
     size_t buffer_size;
     pthread_mutex_t buffer_lock;
+    double audio_latency;
 
     int cfg_frames_per_buffer;
-    int cfg_sample_rate;
 };
 
 static const int fmtmap[][2] = {
@@ -69,9 +69,8 @@ static void uninit(struct ao *ao)
 
     pthread_mutex_destroy(&p->buffer_lock);
 
-    free(p->buf1);
-    free(p->buf2);
-    p->curbuf = p->buf1 = p->buf2 = NULL;
+    free(p->buf);
+    p->buf = NULL;
     p->buffer_size = 0;
 }
 
@@ -87,21 +86,20 @@ static void buffer_callback(SLBufferQueueItf buffer_queue, void *context)
 
     pthread_mutex_lock(&p->buffer_lock);
 
-    data[0] = p->curbuf;
+    data[0] = p->buf;
     delay = 2 * p->buffer_size / (double)ao->bps;
+    delay += p->audio_latency;
     ao_read_data(ao, data, p->buffer_size / ao->sstride,
         mp_time_us() + 1000000LL * delay);
 
-    res = (*buffer_queue)->Enqueue(buffer_queue, p->curbuf, p->buffer_size);
+    res = (*buffer_queue)->Enqueue(buffer_queue, p->buf, p->buffer_size);
     if (res != SL_RESULT_SUCCESS)
         MP_ERR(ao, "Failed to Enqueue: %d\n", res);
-    else
-        p->curbuf = (p->curbuf == p->buf1) ? p->buf2 : p->buf1;
 
     pthread_mutex_unlock(&p->buffer_lock);
 }
 
-#define DEFAULT_BUFFER_SIZE_MS 50
+#define DEFAULT_BUFFER_SIZE_MS 250
 
 #define CHK(stmt) \
     { \
@@ -131,12 +129,12 @@ static int init(struct ao *ao)
     CHK((*p->output_mix)->Realize(p->output_mix, SL_BOOLEAN_FALSE));
 
     locator_buffer_queue.locatorType = SL_DATALOCATOR_BUFFERQUEUE;
-    locator_buffer_queue.numBuffers = 2;
+    locator_buffer_queue.numBuffers = 1;
 
     pcm.formatType = SL_DATAFORMAT_PCM;
     pcm.numChannels = 2;
 
-    int compatible_formats[AF_FORMAT_COUNT];
+    int compatible_formats[AF_FORMAT_COUNT + 1];
     af_get_best_sample_formats(ao->format, compatible_formats);
     pcm.bitsPerSample = 0;
     for (int i = 0; compatible_formats[i] && !pcm.bitsPerSample; ++i)
@@ -154,9 +152,6 @@ static int init(struct ao *ao)
     pcm.channelMask = SL_SPEAKER_FRONT_LEFT | SL_SPEAKER_FRONT_RIGHT;
     pcm.endianness = SL_BYTEORDER_LITTLEENDIAN;
 
-    if (p->cfg_sample_rate)
-        ao->samplerate = p->cfg_sample_rate;
-
     // samplesPerSec is misnamed, actually it's samples per ms
     pcm.samplesPerSec = ao->samplerate * 1000;
 
@@ -166,10 +161,8 @@ static int init(struct ao *ao)
         ao->device_buffer = ao->samplerate * DEFAULT_BUFFER_SIZE_MS / 1000;
     p->buffer_size = ao->device_buffer * ao->channels.num *
         af_fmt_to_bytes(ao->format);
-    p->buf1 = calloc(1, p->buffer_size);
-    p->buf2 = calloc(1, p->buffer_size);
-    p->curbuf = p->buf1;
-    if (!p->buf1 || !p->buf2) {
+    p->buf = calloc(1, p->buffer_size);
+    if (!p->buf) {
         MP_ERR(ao, "Failed to allocate device buffer\n");
         goto error;
     }
@@ -188,16 +181,41 @@ static int init(struct ao *ao)
     audio_sink.pLocator = (void*)&locator_output_mix;
     audio_sink.pFormat = NULL;
 
-    SLboolean required[] = { SL_BOOLEAN_TRUE };
-    SLInterfaceID iid_array[] = { SL_IID_BUFFERQUEUE };
+    SLInterfaceID iid_array[] = { SL_IID_BUFFERQUEUE, SL_IID_ANDROIDCONFIGURATION };
+    SLboolean required[] = { SL_BOOLEAN_TRUE, SL_BOOLEAN_FALSE };
     CHK((*p->engine)->CreateAudioPlayer(p->engine, &p->player, &audio_source,
-        &audio_sink, 1, iid_array, required));
+        &audio_sink, 2, iid_array, required));
+
     CHK((*p->player)->Realize(p->player, SL_BOOLEAN_FALSE));
     CHK((*p->player)->GetInterface(p->player, SL_IID_PLAY, (void*)&p->play));
     CHK((*p->player)->GetInterface(p->player, SL_IID_BUFFERQUEUE,
         (void*)&p->buffer_queue));
     CHK((*p->buffer_queue)->RegisterCallback(p->buffer_queue,
         buffer_callback, ao));
+    CHK((*p->play)->SetPlayState(p->play, SL_PLAYSTATE_PLAYING));
+
+    SLAndroidConfigurationItf android_config;
+    SLuint32 audio_latency = 0, value_size = sizeof(SLuint32);
+
+    SLint32 get_interface_result = (*p->player)->GetInterface(
+        p->player,
+        SL_IID_ANDROIDCONFIGURATION,
+        &android_config
+    );
+
+    if (get_interface_result == SL_RESULT_SUCCESS) {
+        SLint32 get_configuration_result = (*android_config)->GetConfiguration(
+            android_config,
+            (const SLchar *)"androidGetAudioLatency",
+            &value_size,
+            &audio_latency
+        );
+
+        if (get_configuration_result == SL_RESULT_SUCCESS) {
+            p->audio_latency = (double)audio_latency / 1000.0;
+            MP_INFO(ao, "Device latency is %f\n", p->audio_latency);
+        }
+    }
 
     return 1;
 error:
@@ -207,26 +225,15 @@ error:
 
 #undef CHK
 
-static void set_play_state(struct ao *ao, SLuint32 state)
-{
-    struct priv *p = ao->priv;
-    SLresult res = (*p->play)->SetPlayState(p->play, state);
-    if (res != SL_RESULT_SUCCESS)
-        MP_ERR(ao, "Failed to SetPlayState(%d): %d\n", state, res);
-}
-
 static void reset(struct ao *ao)
 {
-    set_play_state(ao, SL_PLAYSTATE_STOPPED);
+    struct priv *p = ao->priv;
+    (*p->buffer_queue)->Clear(p->buffer_queue);
 }
 
 static void resume(struct ao *ao)
 {
     struct priv *p = ao->priv;
-    set_play_state(ao, SL_PLAYSTATE_PLAYING);
-
-    // enqueue two buffers
-    buffer_callback(p->buffer_queue, ao);
     buffer_callback(p->buffer_queue, ao);
 }
 
@@ -242,8 +249,7 @@ const struct ao_driver audio_out_opensles = {
 
     .priv_size = sizeof(struct priv),
     .options = (const struct m_option[]) {
-        OPT_INTRANGE("frames-per-buffer", cfg_frames_per_buffer, 0, 1, 10000),
-        OPT_INTRANGE("sample-rate", cfg_sample_rate, 0, 1000, 100000),
+        OPT_INTRANGE("frames-per-buffer", cfg_frames_per_buffer, 0, 1, 96000),
         {0}
     },
     .options_prefix = "opensles",
