@@ -61,9 +61,6 @@ struct ao {
     // default device should be used, this is set to NULL.
     char *device;
 
-    // Device actually chosen by the AO
-    char *detected_device;
-
     // Application name to report to the audio API.
     char *client_name;
 
@@ -73,6 +70,9 @@ struct ao {
     // Internal events (use ao_request_reload(), ao_hotplug_event())
     atomic_int events_;
 
+    // Float gain multiplicator
+    mp_atomic_float gain;
+
     int buffer;
     double def_buffer;
     void *api_priv;
@@ -130,6 +130,10 @@ struct ao_driver {
     const char *name;
     // Description shown with --ao=help.
     const char *description;
+    // This requires waiting for a AO_EVENT_INITIAL_UNBLOCK event before the
+    // first play() call is done. Encode mode uses this, and push mode
+    // respects it automatically (don't use with pull mode).
+    bool initially_blocked;
     // Init the device using ao->format/ao->channels/ao->samplerate. If the
     // device doesn't accept these parameters, you can attempt to negotiate
     // fallback parameters, and set the ao format fields accordingly.
@@ -215,6 +219,8 @@ bool ao_chmap_sel_get_def(struct ao *ao, const struct mp_chmap_sel *s,
 void ao_device_list_add(struct ao_device_list *list, struct ao *ao,
                         struct ao_device_desc *e);
 
+void ao_post_process_data(struct ao *ao, void **data, int num_samples);
+
 struct ao_convert_fmt {
     int src_fmt;        // source AF_FORMAT_*
     int channels;       // number of channels
