@@ -5,20 +5,18 @@
  *
  * This file is part of mpv.
  *
- * mpv is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * mpv is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with mpv.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Almost LGPL.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -44,10 +42,9 @@
 #include <libavdevice/avdevice.h>
 #endif
 
-#if HAVE_IS_LIBAV
+#if HAVE_LIBAV
 #include <libavresample/avresample.h>
-#endif
-#if HAVE_IS_FFMPEG
+#else
 #include <libswresample/swresample.h>
 #endif
 
@@ -67,9 +64,9 @@ static bool log_print_prefix = true;
 static int av_log_level_to_mp_level(int av_level)
 {
     if (av_level > AV_LOG_VERBOSE)
-        return MSGL_DEBUG;
+        return MSGL_TRACE;
     if (av_level > AV_LOG_INFO)
-        return MSGL_V;
+        return MSGL_DEBUG;
     if (av_level > AV_LOG_WARNING)
         return MSGL_V;
     if (av_level > AV_LOG_ERROR)
@@ -96,10 +93,10 @@ static struct mp_log *get_av_log(void *ptr)
         AVCodecContext *s = ptr;
         if (s->codec) {
             if (s->codec->type == AVMEDIA_TYPE_AUDIO) {
-                if (s->codec->decode)
+                if (av_codec_is_decoder(s->codec))
                     return log_decaudio;
             } else if (s->codec->type == AVMEDIA_TYPE_VIDEO) {
-                if (s->codec->decode)
+                if (av_codec_is_decoder(s->codec))
                     return log_decvideo;
             }
         }
@@ -162,10 +159,7 @@ void init_libav(struct mpv_global *global)
     }
     pthread_mutex_unlock(&log_lock);
 
-    avcodec_register_all();
-    av_register_all();
     avformat_network_init();
-    avfilter_register_all();
 
 #if HAVE_LIBAVDEVICE
     avdevice_register_all();
@@ -199,10 +193,9 @@ bool print_libav_versions(struct mp_log *log, int v)
         {"libavformat",   LIBAVFORMAT_VERSION_INT,   avformat_version()},
         {"libswscale",    LIBSWSCALE_VERSION_INT,    swscale_version()},
         {"libavfilter",   LIBAVFILTER_VERSION_INT,   avfilter_version()},
-#if HAVE_IS_LIBAV
+#if HAVE_LIBAV
         {"libavresample", LIBAVRESAMPLE_VERSION_INT, avresample_version()},
-#endif
-#if HAVE_IS_FFMPEG
+#else
         {"libswresample", LIBSWRESAMPLE_VERSION_INT, swresample_version()},
 #endif
     };
