@@ -45,6 +45,8 @@ struct sh_stream {
     char *lang;                 // language code
     bool default_track;         // container default track flag
     bool forced_track;          // container forced track flag
+    bool dependent_track;       // container dependent track flag
+    bool still_image;           // video stream contains still images
     int hls_bitrate;
 
     struct mp_tags *tags;
@@ -76,6 +78,11 @@ struct mp_codec_params {
     // Timestamp granularity for converting double<->rational timestamps.
     int native_tb_num, native_tb_den;
 
+    // Used by an obscure bug workaround mechanism. As an exception to the usual
+    // rules, demuxers are allowed to set this after adding the sh_stream, but
+    // only before the demuxer open call returns.
+    struct demux_packet *first_packet;
+
     // STREAM_AUDIO
     int samplerate;
     struct mp_chmap channels;
