@@ -34,7 +34,6 @@
 #include "options/path.h"
 #include "video/mp_image.h"
 #include "video/mp_image_pool.h"
-#include "video/decode/dec_video.h"
 #include "video/out/vo.h"
 #include "video/image_writer.h"
 #include "sub/osd.h"
@@ -364,6 +363,11 @@ static char *gen_fname(screenshot_ctx *ctx, const char *file_ext)
             talloc_free(t);
         }
 
+        char *full_dir = bstrto0(fname, mp_dirname(fname));
+        if (!mp_path_exists(full_dir)) {
+            mp_mkdirp(full_dir);
+        }
+
         if (!mp_path_exists(fname))
             return fname;
 
@@ -385,16 +389,30 @@ static void add_subs(struct MPContext *mpctx, struct mp_image *image)
                       OSD_DRAW_SUB_ONLY, image);
 }
 
-static struct mp_image *screenshot_get(struct MPContext *mpctx, int mode)
+static struct mp_image *screenshot_get(struct MPContext *mpctx, int mode,
+                                       bool high_depth)
 {
     struct mp_image *image = NULL;
     if (mode == MODE_SUBTITLES && osd_get_render_subs_in_filter(mpctx->osd))
         mode = 0;
+    bool need_add_subs = mode == MODE_SUBTITLES;
 
     if (mpctx->video_out && mpctx->video_out->config_ok) {
         vo_wait_frame(mpctx->video_out); // important for each-frame mode
 
-        if (mode != MODE_FULL_WINDOW)
+        struct voctrl_screenshot ctrl = {
+            .scaled = mode == MODE_FULL_WINDOW,
+            .subs = mode != 0,
+            .osd = mode == MODE_FULL_WINDOW,
+            .high_bit_depth = high_depth &&
+                              mpctx->opts->screenshot_image_opts->high_bit_depth,
+        };
+        vo_control(mpctx->video_out, VOCTRL_SCREENSHOT, &ctrl);
+        image = ctrl.res;
+        if (image)
+            need_add_subs = false;
+
+        if (!image && mode != MODE_FULL_WINDOW)
             image = vo_get_current_frame(mpctx->video_out);
         if (!image) {
             vo_control(mpctx->video_out, VOCTRL_SCREENSHOT_WIN, &image);
@@ -404,17 +422,11 @@ static struct mp_image *screenshot_get(struct MPContext *mpctx, int mode)
 
     if (image && (image->fmt.flags & MP_IMGFLAG_HWACCEL)) {
         struct mp_image *nimage = mp_image_hw_download(image, NULL);
-        if (!nimage && mpctx->vo_chain && mpctx->vo_chain->hwdec_devs) {
-            struct mp_hwdec_ctx *ctx =
-                hwdec_devices_get_first(mpctx->vo_chain->hwdec_devs);
-            if (ctx && ctx->download_image)
-                nimage = ctx->download_image(ctx, image, NULL);
-        }
         talloc_free(image);
         image = nimage;
     }
 
-    if (image && mode == MODE_SUBTITLES)
+    if (image && need_add_subs)
         add_subs(mpctx, image);
 
     return image;
@@ -422,7 +434,7 @@ static struct mp_image *screenshot_get(struct MPContext *mpctx, int mode)
 
 struct mp_image *screenshot_get_rgb(struct MPContext *mpctx, int mode)
 {
-    struct mp_image *mpi = screenshot_get(mpctx, mode);
+    struct mp_image *mpi = screenshot_get(mpctx, mode, false);
     if (!mpi)
         return NULL;
     struct mp_image *res = convert_image(mpi, IMGFMT_BGR0, mpctx->log);
@@ -442,7 +454,8 @@ void screenshot_to_file(struct MPContext *mpctx, const char *filename, int mode,
     int format = image_writer_format_from_ext(ext);
     if (format)
         opts.format = format;
-    struct mp_image *image = screenshot_get(mpctx, mode);
+    bool high_depth = image_writer_high_depth(&opts);
+    struct mp_image *image = screenshot_get(mpctx, mode, high_depth);
     if (!image) {
         screenshot_msg(ctx, MSGL_ERR, "Taking screenshot failed.");
         goto end;
@@ -473,10 +486,12 @@ void screenshot_request(struct MPContext *mpctx, int mode, bool each_frame,
     ctx->mode = mode;
     ctx->osd = osd;
 
-    struct mp_image *image = screenshot_get(mpctx, mode);
+    struct image_writer_opts *opts = mpctx->opts->screenshot_image_opts;
+    bool high_depth = image_writer_high_depth(opts);
+
+    struct mp_image *image = screenshot_get(mpctx, mode, high_depth);
 
     if (image) {
-        struct image_writer_opts *opts = mpctx->opts->screenshot_image_opts;
         char *filename = gen_fname(ctx, image_writer_file_ext(opts));
         if (filename)
             write_screenshot(mpctx, image, filename, NULL, async);
