@@ -49,7 +49,6 @@ struct sd_ass_priv {
     char last_text[500];
     struct mp_image_params video_params;
     struct mp_image_params last_params;
-    double sub_speed, video_fps, frame_fps;
     int64_t *seen_packets;
     int num_seen_packets;
     bool duration_unknown;
@@ -60,7 +59,7 @@ static void fill_plaintext(struct sd *sd, double pts);
 
 // Add default styles, if the track does not have any styles yet.
 // Apply style overrides if the user provides any.
-static void mp_ass_add_default_styles(ASS_Track *track, struct MPOpts *opts)
+static void mp_ass_add_default_styles(ASS_Track *track, struct mp_subtitle_opts *opts)
 {
     if (opts->ass_styles_file && opts->ass_style_override)
         ass_read_styles(track, opts->ass_styles_file, NULL);
@@ -87,10 +86,14 @@ static const char *const font_mimetypes[] = {
     "application/vnd.ms-opentype",
     "application/x-font-ttf",
     "application/x-font", // probably incorrect
+    "font/collection",
+    "font/otf",
+    "font/sfnt",
+    "font/ttf",
     NULL
 };
 
-static const char *const font_exts[] = {".ttf", ".ttc", ".otf", NULL};
+static const char *const font_exts[] = {".ttf", ".ttc", ".otf", ".otc", NULL};
 
 static bool attachment_is_font(struct mp_log *log, struct demux_attachment *f)
 {
@@ -117,7 +120,7 @@ static bool attachment_is_font(struct mp_log *log, struct demux_attachment *f)
 static void add_subtitle_fonts(struct sd *sd)
 {
     struct sd_ass_priv *ctx = sd->priv;
-    struct MPOpts *opts = sd->opts;
+    struct mp_subtitle_opts *opts = sd->opts;
     if (!opts->ass_enabled || !opts->use_embedded_fonts || !sd->attachments)
         return;
     for (int i = 0; i < sd->attachments->num_entries; i++) {
@@ -143,27 +146,9 @@ static void enable_output(struct sd *sd, bool enable)
     }
 }
 
-static void update_subtitle_speed(struct sd *sd)
-{
-    struct MPOpts *opts = sd->opts;
-    struct sd_ass_priv *ctx = sd->priv;
-    ctx->sub_speed = 1.0;
-
-    if (ctx->video_fps > 0 && ctx->frame_fps > 0) {
-        MP_VERBOSE(sd, "Frame based format, dummy FPS: %f, video FPS: %f\n",
-                   ctx->frame_fps, ctx->video_fps);
-        ctx->sub_speed *= ctx->frame_fps / ctx->video_fps;
-    }
-
-    if (opts->sub_fps && ctx->video_fps)
-        ctx->sub_speed *= opts->sub_fps / ctx->video_fps;
-
-    ctx->sub_speed *= opts->sub_speed;
-}
-
 static int init(struct sd *sd)
 {
-    struct MPOpts *opts = sd->opts;
+    struct mp_subtitle_opts *opts = sd->opts;
     struct sd_ass_priv *ctx = talloc_zero(sd, struct sd_ass_priv);
     sd->priv = ctx;
 
@@ -184,6 +169,7 @@ static int init(struct sd *sd)
     }
 
     ctx->ass_library = mp_ass_init(sd->global, sd->log);
+    ass_set_extract_fonts(ctx->ass_library, opts->use_embedded_fonts);
 
     add_subtitle_fonts(sd);
 
@@ -208,9 +194,6 @@ static int init(struct sd *sd)
     ass_set_check_readorder(ctx->ass_track, sd->opts->sub_clear_on_seek ? 0 : 1);
 #endif
 
-    ctx->frame_fps = sd->codec->frame_based;
-    update_subtitle_speed(sd);
-
     enable_output(sd, true);
 
     ctx->packer = mp_ass_packer_alloc(ctx);
@@ -297,7 +280,7 @@ static void decode(struct sd *sd, struct demux_packet *packet)
 static void configure_ass(struct sd *sd, struct mp_osd_res *dim,
                           bool converted, ASS_Track *track)
 {
-    struct MPOpts *opts = sd->opts;
+    struct mp_subtitle_opts *opts = sd->opts;
     struct sd_ass_priv *ctx = sd->priv;
     ASS_Renderer *priv = ctx->ass_renderer;
 
@@ -383,8 +366,6 @@ static long long find_timestamp(struct sd *sd, double pts)
     if (pts == MP_NOPTS_VALUE)
         return 0;
 
-    pts /= priv->sub_speed;
-
     long long ts = llrint(pts * 1000);
 
     if (!sd->opts->sub_fix_timing || sd->opts->ass_style_override == 0)
@@ -448,7 +429,7 @@ static void get_bitmaps(struct sd *sd, struct mp_osd_res dim, int format,
                         double pts, struct sub_bitmaps *res)
 {
     struct sd_ass_priv *ctx = sd->priv;
-    struct MPOpts *opts = sd->opts;
+    struct mp_subtitle_opts *opts = sd->opts;
     bool no_ass = !opts->ass_enabled || ctx->on_top ||
                   opts->ass_style_override == 5;
     bool converted = ctx->is_converted || no_ass;
@@ -675,11 +656,11 @@ static int control(struct sd *sd, enum sd_ctrl cmd, void *arg)
     switch (cmd) {
     case SD_CTRL_SUB_STEP: {
         double *a = arg;
-        long long ts = llrint(a[0] * (1000.0 / ctx->sub_speed));
+        long long ts = llrint(a[0] * 1000.0);
         long long res = ass_step_sub(ctx->ass_track, ts, a[1]);
         if (!res)
             return false;
-        a[0] = res / (1000.0 / ctx->sub_speed);
+        a[0] += res / 1000.0;
         return true;
     }
     case SD_CTRL_SET_VIDEO_PARAMS:
@@ -688,13 +669,6 @@ static int control(struct sd *sd, enum sd_ctrl cmd, void *arg)
     case SD_CTRL_SET_TOP:
         ctx->on_top = *(bool *)arg;
         return CONTROL_OK;
-    case SD_CTRL_SET_VIDEO_DEF_FPS:
-        ctx->video_fps = *(double *)arg;
-        update_subtitle_speed(sd);
-        return CONTROL_OK;
-    case SD_CTRL_UPDATE_SPEED:
-        update_subtitle_speed(sd);
-        return CONTROL_OK;
     default:
         return CONTROL_UNKNOWN;
     }
@@ -716,7 +690,7 @@ const struct sd_functions sd_ass = {
 // Disgusting hack for (xy-)vsfilter color compatibility.
 static void mangle_colors(struct sd *sd, struct sub_bitmaps *parts)
 {
-    struct MPOpts *opts = sd->opts;
+    struct mp_subtitle_opts *opts = sd->opts;
     struct sd_ass_priv *ctx = sd->priv;
     enum mp_csp csp = 0;
     enum mp_csp_levels levels = 0;
