@@ -57,25 +57,28 @@ static const struct {
     {IMGFMT_0BGR,  AV_PIX_FMT_ABGR},
 #endif
 
+    {IMGFMT_RGBA64, AV_PIX_FMT_RGBA64},
+
     {IMGFMT_VDPAU, AV_PIX_FMT_VDPAU},
 #if HAVE_VIDEOTOOLBOX_HWACCEL
     {IMGFMT_VIDEOTOOLBOX,   AV_PIX_FMT_VIDEOTOOLBOX},
 #endif
-    {IMGFMT_VAAPI, AV_PIX_FMT_VAAPI_VLD},
+#if HAVE_ANDROID
+    {IMGFMT_MEDIACODEC, AV_PIX_FMT_MEDIACODEC},
+#endif
+    {IMGFMT_VAAPI, AV_PIX_FMT_VAAPI},
     {IMGFMT_DXVA2, AV_PIX_FMT_DXVA2_VLD},
 #if HAVE_D3D_HWACCEL
-#if HAVE_D3D_HWACCEL_NEW
-    {IMGFMT_D3D11VA, AV_PIX_FMT_D3D11},
-    {IMGFMT_D3D11NV12, AV_PIX_FMT_D3D11},
-#else
-    {IMGFMT_D3D11VA, AV_PIX_FMT_D3D11VA_VLD},
-#endif
+    {IMGFMT_D3D11, AV_PIX_FMT_D3D11},
 #endif
     {IMGFMT_MMAL, AV_PIX_FMT_MMAL},
 #if HAVE_CUDA_HWACCEL
     {IMGFMT_CUDA, AV_PIX_FMT_CUDA},
 #endif
     {IMGFMT_P010, AV_PIX_FMT_P010},
+#if HAVE_DRMPRIME
+    {IMGFMT_DRMPRIME, AV_PIX_FMT_DRM_PRIME},
+#endif
 
     {0, AV_PIX_FMT_NONE}
 };
