@@ -45,7 +45,8 @@
 @synthesize targetScreen = _target_screen;
 @synthesize previousScreen = _previous_screen;
 @synthesize currentScreen = _current_screen;
-@synthesize unfScreen = _unf_Screen;
+@synthesize unfScreen = _unf_screen;
+
 - (id)initWithContentRect:(NSRect)content_rect
                 styleMask:(NSWindowStyleMask)style_mask
                   backing:(NSBackingStoreType)buffering_type
@@ -386,8 +387,11 @@
 
 - (NSRect)constrainFrameRect:(NSRect)nf toScreen:(NSScreen *)screen
 {
-    if (_is_animating && ![self.adapter isInFullScreenMode])
+    if ((_is_animating && ![self.adapter isInFullScreenMode]) ||
+        (!_is_animating && [self.adapter isInFullScreenMode]))
+    {
         return nf;
+    }
 
     screen = screen ?: self.screen ?: [NSScreen mainScreen];
     NSRect of  = [self frame];
