@@ -61,12 +61,6 @@ class CocoaCB: NSObject {
         if backendState == .uninitialized {
             backendState = .needsInit
 
-            if let app = NSApp as? Application {
-                let ptr = mp_get_config_group(mpv.mpctx!, vo.pointee.global,
-                                              app.getMacOSConf())
-                mpv.macOpts = UnsafeMutablePointer<macos_opts>(OpaquePointer(ptr))!.pointee
-            }
-
             view = EventsView(cocoaCB: self)
             view.layer = layer
             view.wantsLayer = true
@@ -241,7 +235,9 @@ class CocoaCB: NSObject {
 
     func updateICCProfile() {
         mpv.setRenderICCProfile(window.screen!.colorSpace!)
-        layer.colorspace = window.screen!.colorSpace!.cgColorSpace!
+        if #available(macOS 10.11, *) {
+            layer.colorspace = window.screen!.colorSpace!.cgColorSpace!
+        }
     }
 
     func lmuToLux(_ v: UInt64) -> Int {
