@@ -24,13 +24,14 @@
 
 #include "angle_dynamic.h"
 #include "egl_helpers.h"
-#include "d3d11_helpers.h"
+#include "video/out/gpu/d3d11_helpers.h"
 
 #include "common/common.h"
 #include "options/m_config.h"
 #include "video/out/w32_common.h"
 #include "osdep/windows_utils.h"
 #include "context.h"
+#include "utils.h"
 
 #ifndef EGL_D3D_TEXTURE_ANGLE
 #define EGL_D3D_TEXTURE_ANGLE 0x33A3
@@ -52,8 +53,6 @@ struct angle_opts {
     int d3d11_warp;
     int d3d11_feature_level;
     int egl_windowing;
-    int swapchain_length; // Currently only works with DXGI 1.2+
-    int max_frame_latency;
     int flip;
 };
 
@@ -77,9 +76,9 @@ const struct m_sub_options angle_conf = {
                    ({"auto", -1},
                     {"no", 0},
                     {"yes", 1})),
-        OPT_INTRANGE("angle-swapchain-length", swapchain_length, 0, 2, 16),
-        OPT_INTRANGE("angle-max-frame-latency", max_frame_latency, 0, 1, 16),
         OPT_FLAG("angle-flip", flip, 0),
+        OPT_REPLACED("angle-max-frame-latency", "swapchain-depth"),
+        OPT_REMOVED("angle-swapchain-length", "controlled by --swapchain-depth"),
         {0}
     },
     .defaults = &(const struct angle_opts) {
@@ -87,14 +86,14 @@ const struct m_sub_options angle_conf = {
         .d3d11_warp = -1,
         .d3d11_feature_level = D3D_FEATURE_LEVEL_11_0,
         .egl_windowing = -1,
-        .swapchain_length = 6,
-        .max_frame_latency = 3,
         .flip = 1,
     },
     .size = sizeof(struct angle_opts),
 };
 
 struct priv {
+    GL gl;
+
     IDXGISwapChain *dxgi_swapchain;
 
     ID3D11Device *d3d11_device;
@@ -110,20 +109,21 @@ struct priv {
 
     int sc_width, sc_height; // Swap chain width and height
     int swapinterval;
+    bool flipped;
 
     struct angle_opts *opts;
 };
 
-static __thread struct MPGLContext *current_ctx;
+static __thread struct ra_ctx *current_ctx;
 
-static void update_sizes(MPGLContext *ctx)
+static void update_sizes(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     p->sc_width = ctx->vo->dwidth ? ctx->vo->dwidth : 1;
     p->sc_height = ctx->vo->dheight ? ctx->vo->dheight : 1;
 }
 
-static void d3d11_backbuffer_release(MPGLContext *ctx)
+static void d3d11_backbuffer_release(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
@@ -137,7 +137,7 @@ static void d3d11_backbuffer_release(MPGLContext *ctx)
     SAFE_RELEASE(p->d3d11_backbuffer);
 }
 
-static bool d3d11_backbuffer_get(MPGLContext *ctx)
+static bool d3d11_backbuffer_get(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -168,7 +168,7 @@ static bool d3d11_backbuffer_get(MPGLContext *ctx)
     return true;
 }
 
-static void d3d11_backbuffer_resize(MPGLContext *ctx)
+static void d3d11_backbuffer_resize(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -197,7 +197,7 @@ static void d3d11_backbuffer_resize(MPGLContext *ctx)
         MP_FATAL(vo, "Couldn't get back buffer after resize\n");
 }
 
-static void d3d11_device_destroy(MPGLContext *ctx)
+static void d3d11_device_destroy(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
@@ -215,7 +215,7 @@ static void d3d11_device_destroy(MPGLContext *ctx)
     SAFE_RELEASE(p->d3d11_device);
 }
 
-static bool d3d11_device_create(MPGLContext *ctx, int flags)
+static bool d3d11_device_create(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -226,7 +226,7 @@ static bool d3d11_device_create(MPGLContext *ctx, int flags)
         .force_warp = o->d3d11_warp == 1,
         .max_feature_level = o->d3d11_feature_level,
         .min_feature_level = D3D_FEATURE_LEVEL_9_3,
-        .max_frame_latency = o->max_frame_latency,
+        .max_frame_latency = ctx->opts.swapchain_depth,
     };
     if (!mp_d3d11_create_present_device(vo->log, &device_opts, &p->d3d11_device))
         return false;
@@ -262,7 +262,7 @@ static bool d3d11_device_create(MPGLContext *ctx, int flags)
     return true;
 }
 
-static void d3d11_swapchain_surface_destroy(MPGLContext *ctx)
+static void d3d11_swapchain_surface_destroy(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
@@ -277,7 +277,7 @@ static void d3d11_swapchain_surface_destroy(MPGLContext *ctx)
         ID3D11DeviceContext_Flush(p->d3d11_context);
 }
 
-static bool d3d11_swapchain_surface_create(MPGLContext *ctx, int flags)
+static bool d3d11_swapchain_surface_create(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -292,7 +292,9 @@ static bool d3d11_swapchain_surface_create(MPGLContext *ctx, int flags)
         .width = p->sc_width,
         .height = p->sc_height,
         .flip = o->flip,
-        .length = o->swapchain_length,
+        // Add one frame for the backbuffer and one frame of "slack" to reduce
+        // contention with the window manager when acquiring the backbuffer
+        .length = ctx->opts.swapchain_depth + 2,
         .usage = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_SHADER_INPUT,
     };
     if (!mp_d3d11_create_swapchain(p->d3d11_device, vo->log, &swapchain_opts,
@@ -301,8 +303,7 @@ static bool d3d11_swapchain_surface_create(MPGLContext *ctx, int flags)
     if (!d3d11_backbuffer_get(ctx))
         goto fail;
 
-    // EGL_D3D_TEXTURE_ANGLE pbuffers are always flipped vertically
-    ctx->flip_v = true;
+    p->flipped = true;
     return true;
 
 fail:
@@ -310,7 +311,7 @@ fail:
     return false;
 }
 
-static void d3d9_device_destroy(MPGLContext *ctx)
+static void d3d9_device_destroy(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
@@ -319,7 +320,7 @@ static void d3d9_device_destroy(MPGLContext *ctx)
     p->egl_display = EGL_NO_DISPLAY;
 }
 
-static bool d3d9_device_create(MPGLContext *ctx, int flags)
+static bool d3d9_device_create(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -348,7 +349,7 @@ static bool d3d9_device_create(MPGLContext *ctx, int flags)
     return true;
 }
 
-static void egl_window_surface_destroy(MPGLContext *ctx)
+static void egl_window_surface_destroy(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     if (p->egl_window) {
@@ -357,7 +358,7 @@ static void egl_window_surface_destroy(MPGLContext *ctx)
     }
 }
 
-static bool egl_window_surface_create(MPGLContext *ctx, int flags)
+static bool egl_window_surface_create(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -374,7 +375,7 @@ static bool egl_window_surface_create(MPGLContext *ctx, int flags)
                 EGL_SURFACE_ORIENTATION_ANGLE);
             MP_TARRAY_APPEND(NULL, window_attribs, window_attribs_len,
                 EGL_SURFACE_ORIENTATION_INVERT_Y_ANGLE);
-            ctx->flip_v = true;
+            p->flipped = true;
             MP_VERBOSE(vo, "Rendering flipped.\n");
         }
     }
@@ -396,7 +397,7 @@ fail:
     return false;
 }
 
-static void context_destroy(struct MPGLContext *ctx)
+static void context_destroy(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     if (p->egl_context) {
@@ -407,7 +408,7 @@ static void context_destroy(struct MPGLContext *ctx)
     p->egl_context = EGL_NO_CONTEXT;
 }
 
-static bool context_init(struct MPGLContext *ctx, int flags)
+static bool context_init(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -421,8 +422,8 @@ static bool context_init(struct MPGLContext *ctx, int flags)
     if (exts)
         MP_DBG(vo, "EGL extensions: %s\n", exts);
 
-    if (!mpegl_create_context(p->egl_display, vo->log, flags | VOFLAG_GLES,
-                              &p->egl_context, &p->egl_config))
+    if (!mpegl_create_context(ctx, p->egl_display, &p->egl_context,
+                              &p->egl_config))
     {
         MP_FATAL(vo, "Could not create EGL context!\n");
         goto fail;
@@ -434,10 +435,12 @@ fail:
     return false;
 }
 
-static void angle_uninit(struct MPGLContext *ctx)
+static void angle_uninit(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
+    ra_gl_ctx_uninit(ctx);
+
     DwmEnableMMCSS(FALSE);
 
     // Uninit the EGL surface implementation that is being used. Note: This may
@@ -474,180 +477,7 @@ static int GLAPIENTRY angle_swap_interval(int interval)
     }
 }
 
-static int angle_init(struct MPGLContext *ctx, int flags)
-{
-    struct priv *p = ctx->priv;
-    struct vo *vo = ctx->vo;
-
-    p->opts = mp_get_config_group(ctx, ctx->global, &angle_conf);
-    struct angle_opts *o = p->opts;
-
-    // DWM MMCSS cargo-cult. The dxinterop backend also does this.
-    DwmEnableMMCSS(TRUE);
-
-    if (!angle_load()) {
-        MP_VERBOSE(vo, "Failed to load LIBEGL.DLL\n");
-        goto fail;
-    }
-
-    // Create the underlying EGL device implementation
-    bool context_ok = false;
-    if ((!context_ok && !o->renderer) || o->renderer == RENDERER_D3D11) {
-        context_ok = d3d11_device_create(ctx, flags);
-        if (context_ok) {
-            context_ok = context_init(ctx, flags);
-            if (!context_ok)
-                d3d11_device_destroy(ctx);
-        }
-    }
-    if ((!context_ok && !o->renderer) || o->renderer == RENDERER_D3D9) {
-        context_ok = d3d9_device_create(ctx, flags);
-        if (context_ok) {
-            MP_VERBOSE(vo, "Using Direct3D 9\n");
-
-            context_ok = context_init(ctx, flags);
-            if (!context_ok)
-                d3d9_device_destroy(ctx);
-        }
-    }
-    if (!context_ok)
-        goto fail;
-
-    if (!vo_w32_init(vo))
-        goto fail;
-
-    // Create the underlying EGL surface implementation
-    bool surface_ok = false;
-    if ((!surface_ok && o->egl_windowing == -1) || o->egl_windowing == 0) {
-        surface_ok = d3d11_swapchain_surface_create(ctx, flags);
-    }
-    if ((!surface_ok && o->egl_windowing == -1) || o->egl_windowing == 1) {
-        surface_ok = egl_window_surface_create(ctx, flags);
-        if (surface_ok)
-            MP_VERBOSE(vo, "Using EGL windowing\n");
-    }
-    if (!surface_ok)
-        goto fail;
-
-    mpegl_load_functions(ctx->gl, vo->log);
-
-    current_ctx = ctx;
-    ctx->gl->SwapInterval = angle_swap_interval;
-
-    return 0;
-fail:
-    angle_uninit(ctx);
-    return -1;
-}
-
-static int angle_reconfig(struct MPGLContext *ctx)
-{
-    vo_w32_config(ctx->vo);
-    return 0;
-}
-
-static struct mp_image *d3d11_screenshot(MPGLContext *ctx)
-{
-    struct priv *p = ctx->priv;
-    ID3D11Texture2D *frontbuffer = NULL;
-    ID3D11Texture2D *staging = NULL;
-    struct mp_image *img = NULL;
-    HRESULT hr;
-
-    if (!p->dxgi_swapchain)
-        goto done;
-
-    // Validate the swap chain. This screenshot method will only work on DXGI
-    // 1.2+ flip/sequential swap chains. It's probably not possible at all with
-    // discard swap chains, since by definition, the backbuffer contents is
-    // discarded on Present().
-    DXGI_SWAP_CHAIN_DESC scd;
-    hr = IDXGISwapChain_GetDesc(p->dxgi_swapchain, &scd);
-    if (FAILED(hr))
-        goto done;
-    if (scd.SwapEffect != DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL)
-        goto done;
-
-    // Get the last buffer that was presented with Present(). This should be
-    // the n-1th buffer for a swap chain of length n.
-    hr = IDXGISwapChain_GetBuffer(p->dxgi_swapchain, scd.BufferCount - 1,
-        &IID_ID3D11Texture2D, (void**)&frontbuffer);
-    if (FAILED(hr))
-        goto done;
-
-    D3D11_TEXTURE2D_DESC td;
-    ID3D11Texture2D_GetDesc(frontbuffer, &td);
-    if (td.SampleDesc.Count > 1)
-        goto done;
-
-    // Validate the backbuffer format and convert to an mpv IMGFMT
-    enum mp_imgfmt fmt;
-    switch (td.Format) {
-    case DXGI_FORMAT_B8G8R8A8_UNORM: fmt = IMGFMT_BGR0; break;
-    case DXGI_FORMAT_R8G8B8A8_UNORM: fmt = IMGFMT_RGB0; break;
-    default:
-        goto done;
-    }
-
-    // Create a staging texture based on the frontbuffer with CPU access
-    td.BindFlags = 0;
-    td.MiscFlags = 0;
-    td.CPUAccessFlags = D3D11_CPU_ACCESS_READ;
-    td.Usage = D3D11_USAGE_STAGING;
-    hr = ID3D11Device_CreateTexture2D(p->d3d11_device, &td, 0, &staging);
-    if (FAILED(hr))
-        goto done;
-
-    ID3D11DeviceContext_CopyResource(p->d3d11_context,
-        (ID3D11Resource*)staging, (ID3D11Resource*)frontbuffer);
-
-    // Attempt to map the staging texture to CPU-accessible memory
-    D3D11_MAPPED_SUBRESOURCE lock;
-    hr = ID3D11DeviceContext_Map(p->d3d11_context, (ID3D11Resource*)staging,
-                                 0, D3D11_MAP_READ, 0, &lock);
-    if (FAILED(hr))
-        goto done;
-
-    img = mp_image_alloc(fmt, td.Width, td.Height);
-    if (!img)
-        return NULL;
-    for (int i = 0; i < td.Height; i++) {
-        memcpy(img->planes[0] + img->stride[0] * i,
-               (char*)lock.pData + lock.RowPitch * i, td.Width * 4);
-    }
-
-    ID3D11DeviceContext_Unmap(p->d3d11_context, (ID3D11Resource*)staging, 0);
-
-done:
-    SAFE_RELEASE(frontbuffer);
-    SAFE_RELEASE(staging);
-    return img;
-}
-
-static int angle_control(MPGLContext *ctx, int *events, int request, void *arg)
-{
-    struct priv *p = ctx->priv;
-
-    // Try a D3D11-specific method of taking a window screenshot
-    if (request == VOCTRL_SCREENSHOT_WIN) {
-        struct mp_image *img = d3d11_screenshot(ctx);
-        if (img) {
-            *(struct mp_image **)arg = img;
-            return true;
-        }
-    }
-
-    int r = vo_w32_control(ctx->vo, events, request, arg);
-    if (*events & VO_EVENT_RESIZE) {
-        if (p->dxgi_swapchain)
-            d3d11_backbuffer_resize(ctx);
-        else
-            eglWaitClient(); // Should get ANGLE to resize its swapchain
-    }
-    return r;
-}
-
-static void d3d11_swap_buffers(MPGLContext *ctx)
+static void d3d11_swap_buffers(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
 
@@ -673,13 +503,13 @@ static void d3d11_swap_buffers(MPGLContext *ctx)
     SAFE_RELEASE(dsv);
 }
 
-static void egl_swap_buffers(MPGLContext *ctx)
+static void egl_swap_buffers(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     eglSwapBuffers(p->egl_display, p->egl_window);
 }
 
-static void angle_swap_buffers(MPGLContext *ctx)
+static void angle_swap_buffers(struct ra_ctx *ctx)
 {
     struct priv *p = ctx->priv;
     if (p->dxgi_swapchain)
@@ -688,12 +518,137 @@ static void angle_swap_buffers(MPGLContext *ctx)
         egl_swap_buffers(ctx);
 }
 
-const struct mpgl_driver mpgl_driver_angle = {
+
+static int angle_color_depth(struct ra_swapchain *sw)
+{
+    // Only 8-bit output is supported at the moment
+    return 8;
+}
+
+static bool angle_submit_frame(struct ra_swapchain *sw,
+                               const struct vo_frame *frame)
+{
+    struct priv *p = sw->ctx->priv;
+    bool ret = ra_gl_ctx_submit_frame(sw, frame);
+    if (p->d3d11_context) {
+        // DXGI Present doesn't flush the immediate context, which can make
+        // timers inaccurate, since the end queries might not be sent until the
+        // next frame. Fix this by flushing the context now.
+        ID3D11DeviceContext_Flush(p->d3d11_context);
+    }
+    return ret;
+}
+
+static bool angle_init(struct ra_ctx *ctx)
+{
+    struct priv *p = ctx->priv = talloc_zero(ctx, struct priv);
+    struct vo *vo = ctx->vo;
+    GL *gl = &p->gl;
+
+    p->opts = mp_get_config_group(ctx, ctx->global, &angle_conf);
+    struct angle_opts *o = p->opts;
+
+    if (!angle_load()) {
+        MP_VERBOSE(vo, "Failed to load LIBEGL.DLL\n");
+        goto fail;
+    }
+
+    // Create the underlying EGL device implementation
+    bool context_ok = false;
+    if ((!context_ok && !o->renderer) || o->renderer == RENDERER_D3D11) {
+        context_ok = d3d11_device_create(ctx);
+        if (context_ok) {
+            context_ok = context_init(ctx);
+            if (!context_ok)
+                d3d11_device_destroy(ctx);
+        }
+    }
+    if ((!context_ok && !o->renderer) || o->renderer == RENDERER_D3D9) {
+        context_ok = d3d9_device_create(ctx);
+        if (context_ok) {
+            MP_VERBOSE(vo, "Using Direct3D 9\n");
+
+            context_ok = context_init(ctx);
+            if (!context_ok)
+                d3d9_device_destroy(ctx);
+        }
+    }
+    if (!context_ok)
+        goto fail;
+
+    if (!vo_w32_init(vo))
+        goto fail;
+
+    // Create the underlying EGL surface implementation
+    bool surface_ok = false;
+    if ((!surface_ok && o->egl_windowing == -1) || o->egl_windowing == 0) {
+        surface_ok = d3d11_swapchain_surface_create(ctx);
+    }
+    if ((!surface_ok && o->egl_windowing == -1) || o->egl_windowing == 1) {
+        surface_ok = egl_window_surface_create(ctx);
+        if (surface_ok)
+            MP_VERBOSE(vo, "Using EGL windowing\n");
+    }
+    if (!surface_ok)
+        goto fail;
+
+    mpegl_load_functions(gl, vo->log);
+
+    current_ctx = ctx;
+    gl->SwapInterval = angle_swap_interval;
+
+    // Custom swapchain impl for the D3D11 swapchain-based surface
+    static const struct ra_swapchain_fns dxgi_swapchain_fns = {
+        .color_depth = angle_color_depth,
+        .submit_frame = angle_submit_frame,
+    };
+    struct ra_gl_ctx_params params = {
+        .swap_buffers = angle_swap_buffers,
+        .flipped = p->flipped,
+        .external_swapchain = p->dxgi_swapchain ? &dxgi_swapchain_fns : NULL,
+    };
+
+    if (!ra_gl_ctx_init(ctx, gl, params))
+        goto fail;
+
+    DwmEnableMMCSS(TRUE); // DWM MMCSS cargo-cult. The dxgl backend also does this.
+
+    return true;
+fail:
+    angle_uninit(ctx);
+    return false;
+}
+
+static void resize(struct ra_ctx *ctx)
+{
+    struct priv *p = ctx->priv;
+    if (p->dxgi_swapchain)
+        d3d11_backbuffer_resize(ctx);
+    else
+        eglWaitClient(); // Should get ANGLE to resize its swapchain
+    ra_gl_ctx_resize(ctx->swapchain, ctx->vo->dwidth, ctx->vo->dheight, 0);
+}
+
+static bool angle_reconfig(struct ra_ctx *ctx)
+{
+    vo_w32_config(ctx->vo);
+    resize(ctx);
+    return true;
+}
+
+static int angle_control(struct ra_ctx *ctx, int *events, int request, void *arg)
+{
+    int ret = vo_w32_control(ctx->vo, events, request, arg);
+    if (*events & VO_EVENT_RESIZE)
+        resize(ctx);
+    return ret;
+}
+
+const struct ra_ctx_fns ra_ctx_angle = {
+    .type           = "opengl",
     .name           = "angle",
-    .priv_size      = sizeof(struct priv),
     .init           = angle_init,
     .reconfig       = angle_reconfig,
-    .swap_buffers   = angle_swap_buffers,
     .control        = angle_control,
     .uninit         = angle_uninit,
 };
