@@ -44,7 +44,7 @@
 #include "sub/osd.h"
 
 #include "opengl/ra_gl.h"
-#include "opengl/video.h"
+#include "gpu/video.h"
 
 struct mp_egl_rpi {
     struct mp_log *log;
@@ -261,12 +261,12 @@ static void update_osd(struct vo *vo)
     MP_STATS(vo, "start rpi_osd");
 
     struct vo_frame frame = {0};
-    struct fbodst target = {
+    struct ra_fbo target = {
         .tex = ra_create_wrapped_fb(p->egl.ra, 0, p->osd_res.w, p->osd_res.h),
         .flip = true,
     };
     gl_video_set_osd_pts(p->gl_video, p->osd_pts);
-    gl_video_render_frame(p->gl_video, &frame, target);
+    gl_video_render_frame(p->gl_video, &frame, target, RENDER_FRAME_DEF);
     ra_tex_free(p->egl.ra, &target.tex);
 
     MP_STATS(vo, "stop rpi_osd");
