@@ -4,7 +4,7 @@ from waflib import Utils
 import os
 
 __all__ = ["check_pthreads", "check_iconv", "check_lua",
-           "check_cocoa", "check_openal", "check_wl_protocols"]
+           "check_cocoa", "check_wl_protocols", "check_swift"]
 
 pthreads_program = load_fragment('pthreads.c')
 
@@ -112,13 +112,11 @@ def check_cocoa(ctx, dependency_identifier):
 
     return res
 
-def check_openal(ctx, dependency_identifier):
-    checks = [
-        check_pkg_config('openal', '>= 1.13'),
-        check_statement(['OpenAL/AL.h'], 'int i = AL_VERSION', framework='OpenAL')
-        ]
-
-    for fn in checks:
-        if fn(ctx, dependency_identifier):
+def check_swift(ctx, dependency_identifier):
+    if ctx.env.SWIFT_VERSION:
+        major = int(ctx.env.SWIFT_VERSION.split('.')[0])
+        ctx.add_optional_message(dependency_identifier,
+                                 'version found: ' + ctx.env.SWIFT_VERSION)
+        if major >= 3:
             return True
     return False
